using System;
using System.Collections.Generic;
using Vevo.Domain.Configurations;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Stores;
using Vevo.Shared.Domain;
using Vevo.Shared.SystemServices;
using Vevo.Deluxe.Domain.DataInterfaces;

namespace Vevo.Domain
{
    public enum RepositoryType
    {
        AdminRepository,
        AffiliateOrderRepository,
        AffiliatePaymentRepository,
        AffiliateRepository,
        BannerRepository,
        BlogRepository,
        BlogCommentRepository,
        CartRepository,
        CartItemRepository,
        CategoryRepository,
        ConfigurationRepository,
        ContentMenuItemRepository,
        ContentMenuRepository,
        ContentRepository,
        CountryRepository,
        CouponRepository,
        CultureRepository,
        CurrencyRepository,
        CustomerRepository,
        CustomerRewardPointRepository,
        DepartmentRepository,
        DiscountGroupRepository,
        EmailTemplateDetailRepository,
        EBayListRepository,
        EBayTemplateRepository,
        GiftCertificateRepository,
        GiftRegistryItemRepository,
        GiftRegistryRepository,
        GoogleFeedMappingRepository,
        GoogleFeedTagRepository,
        GoogleFeedTagValueRepository,
        HelpRepository,
        ManufacturerRepository,
        NewsLetterRepository,
        NewsRepository,
        OptionGroupRepository,
        OptionItemRepository,
        OrderRepository,
        OrderItemRepository,
        PaymentLogRepository,
        PaymentOptionRepository,
        PayPalIpnRepository,
        ProductKitGroupRepository,
        ProductRepository,
        PromotionGroupRepository,
        PromotionProductRepository,
        PromotionSubGroupRepository,
        QuickBooksLogRepository,
        RecurringProfileRepository,
        RmaRepository,
        RmaActionRepository,
        ShippingOptionRepository,
        ShippingOptionTypeRepository,
        ShippingWeightRateRepository,
        ShippingOrderTotalRateRepository,
        ShippingZoneGroupRepository,
        SpecificationGroupRepository,
        SpecificationItemRepository,
        SpecificationItemValueRepository,
        StateRepository,
        StoreRepository,
        SubscriptionLevelRepository,
        TaxClassRepository,
        WishListRepository
    }

    public static class DataAccessContext
    {
        #region Private
        private static ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "Configuration", "ConfigurationValue" } );

        private static Dictionary<string, object> _repositories = new Dictionary<string, object>();
        private static Dictionary<RepositoryType, bool> _cacheEnabledStates =
            new Dictionary<RepositoryType, bool>();

        private static ConfigurationCollection _configurations;
        private static ApplicationSettings _applicationSettings;
        private static bool _configEnabled = false;
        private static IStoreRetriever _storeRetriever;

        private static string CreateKey( RepositoryType type, bool caching )
        {
            return type.ToString() + "_" + caching.ToString();
        }

        private static void SetValue( RepositoryType type, bool caching, object repository )
        {
            _repositories[ CreateKey( type, caching ) ] = repository;
        }

        private static object GetValue( RepositoryType type, bool caching )
        {
            return _repositories[ CreateKey( type, caching ) ];
        }

        #endregion


        #region Public Properties

        // This is the global configurations that will be set at the application startup.

        public static ConfigurationCollection Configurations
        {
            get
            {
                try
                {
                    if ( _cacheDependency.getHasChanged )
                    {
                        ClearConfigurationCache();
                    }
                }
                catch ( Exception ex )
                {
                    SaveLogFile.SaveLog( ex );
                }

                if ( _configurations == null || _configurations.Count == 0 )
                {
                    if ( !_configEnabled )
                        _configurations = new ConfigurationCollection();
                    else
                        _configurations = new ConfigurationCollection(
                            ConfigurationRepository.GetAll(),
                            StoreRetriever );
                }

                return _configurations;
            }
        }

        // This is the settings in App_Data\ApplicationSettings.config. This file contains 
        // system configurations that cannot be saved in the database.
        public static ApplicationSettings ApplicationSettings
        {
            get
            {
                if ( _applicationSettings == null )
                    _applicationSettings = new ApplicationSettings( new FileManager() );

                return _applicationSettings;
            }
        }

        public static IStoreRetriever StoreRetriever
        {
            get
            {
                if ( _storeRetriever == null )
                    _storeRetriever = new StoreRetriever();

                return _storeRetriever;
            }
        }

        public static IAdminRepository AdminRepository
        {
            get
            {
                return ( IAdminRepository ) GetValue(
                    RepositoryType.AdminRepository,
                    GetCacheEnabled( RepositoryType.AdminRepository ) );
            }
        }

        public static IAffiliateOrderRepository AffiliateOrderRepository
        {
            get
            {
                return ( IAffiliateOrderRepository ) GetValue(
                    RepositoryType.AffiliateOrderRepository,
                    GetCacheEnabled( RepositoryType.AffiliateOrderRepository ) );
            }
        }

        public static IAffiliatePaymentRepository AffiliatePaymentRepository
        {
            get
            {
                return ( IAffiliatePaymentRepository ) GetValue(
                    RepositoryType.AffiliatePaymentRepository,
                    GetCacheEnabled( RepositoryType.AffiliatePaymentRepository ) );
            }
        }

        public static IAffiliateRepository AffiliateRepository
        {
            get
            {
                return ( IAffiliateRepository ) GetValue(
                    RepositoryType.AffiliateRepository,
                    GetCacheEnabled( RepositoryType.AffiliateRepository ) );
            }
        }

        public static IBannerRepository BannerRepository
        {
            get
            {
                return ( IBannerRepository ) GetValue(
                    RepositoryType.BannerRepository,
                    GetCacheEnabled( RepositoryType.BannerRepository ) );
            }
        }

        public static IBlogRepository BlogRepository
        {
            get
            {
                return ( IBlogRepository ) GetValue(
                    RepositoryType.BlogRepository,
                    GetCacheEnabled( RepositoryType.BlogRepository ) );
            }
        }
        public static IBlogCommentRepository BlogCommentRepository
        {
            get
            {
                return (IBlogCommentRepository) GetValue(
                    RepositoryType.BlogCommentRepository,
                    GetCacheEnabled( RepositoryType.BlogCommentRepository ) );
            }
        }
        public static ICartItemRepository CartItemRepository
        {
            get
            {
                return ( ICartItemRepository ) GetValue(
                    RepositoryType.CartItemRepository,
                    GetCacheEnabled( RepositoryType.CartItemRepository ) );
            }
        }

        public static ICartRepository CartRepository
        {
            get
            {
                return ( ICartRepository ) GetValue(
                    RepositoryType.CartRepository,
                    GetCacheEnabled( RepositoryType.CartRepository ) );
            }
        }

        public static ICategoryRepository CategoryRepository
        {
            get
            {
                return ( ICategoryRepository ) GetValue(
                    RepositoryType.CategoryRepository,
                    GetCacheEnabled( RepositoryType.CategoryRepository ) );
            }
        }

        public static IConfigurationRepository ConfigurationRepository
        {
            get
            {
                return ( IConfigurationRepository ) GetValue(
                    RepositoryType.ConfigurationRepository,
                    GetCacheEnabled( RepositoryType.ConfigurationRepository ) );
            }
        }

        public static IContentMenuItemRepository ContentMenuItemRepository
        {
            get
            {
                return ( IContentMenuItemRepository ) GetValue(
                    RepositoryType.ContentMenuItemRepository,
                    GetCacheEnabled( RepositoryType.ContentMenuItemRepository ) );
            }
        }

        public static IContentMenuRepository ContentMenuRepository
        {
            get
            {
                return ( IContentMenuRepository ) GetValue(
                    RepositoryType.ContentMenuRepository,
                    GetCacheEnabled( RepositoryType.ContentMenuRepository ) );
            }
        }

        public static IContentRepository ContentRepository
        {
            get
            {
                return ( IContentRepository ) GetValue(
                    RepositoryType.ContentRepository,
                    GetCacheEnabled( RepositoryType.ContentRepository ) );
            }
        }


        public static ICountryRepository CountryRepository
        {
            get
            {
                return ( ICountryRepository ) GetValue(
                    RepositoryType.CountryRepository,
                    GetCacheEnabled( RepositoryType.CountryRepository ) );
            }
        }

        public static ICouponRepository CouponRepository
        {
            get
            {
                return ( ICouponRepository ) GetValue(
                    RepositoryType.CouponRepository,
                    GetCacheEnabled( RepositoryType.CouponRepository ) );
            }
        }

        public static ICustomerRepository CustomerRepository
        {
            get
            {
                return ( ICustomerRepository ) GetValue(
                    RepositoryType.CustomerRepository,
                    GetCacheEnabled( RepositoryType.CustomerRepository ) );
            }
        }

        public static ICustomerRewardPointRepository CustomerRewardPointRepository
        {
            get
            {
                return ( ICustomerRewardPointRepository ) GetValue(
                    RepositoryType.CustomerRewardPointRepository,
                    GetCacheEnabled( RepositoryType.CustomerRewardPointRepository ) );
            }
        }

        public static ICultureRepository CultureRepository
        {
            get
            {
                return ( ICultureRepository ) GetValue(
                    RepositoryType.CultureRepository,
                    GetCacheEnabled( RepositoryType.CultureRepository ) );
            }
        }

        public static ICurrencyRepository CurrencyRepository
        {
            get
            {
                return ( ICurrencyRepository ) GetValue(
                    RepositoryType.CurrencyRepository,
                    GetCacheEnabled( RepositoryType.CurrencyRepository ) );
            }
        }

        public static IDepartmentRepository DepartmentRepository
        {
            get
            {
                return ( IDepartmentRepository ) GetValue(
                    RepositoryType.DepartmentRepository,
                    GetCacheEnabled( RepositoryType.DepartmentRepository ) );
            }
        }

        public static IDiscountGroupRepository DiscountGroupRepository
        {
            get
            {
                return ( IDiscountGroupRepository ) GetValue(
                    RepositoryType.DiscountGroupRepository,
                    GetCacheEnabled( RepositoryType.DiscountGroupRepository ) );
            }
        }

        public static IEmailTemplateDetailRepository EmailTemplateDetailRepository
        {
            get
            {
                return ( IEmailTemplateDetailRepository ) GetValue(
                    RepositoryType.EmailTemplateDetailRepository,
                    GetCacheEnabled( RepositoryType.EmailTemplateDetailRepository ) );
            }
        }

        public static IEBayListRepository EBayListRepository
        {
            get
            {
                return ( IEBayListRepository ) GetValue(
                    RepositoryType.EBayListRepository,
                    GetCacheEnabled( RepositoryType.EBayListRepository ) );
            }
        }

        public static IEBayTemplateRepository EBayTemplateRepository
        {
            get
            {
                return ( IEBayTemplateRepository ) GetValue(
                    RepositoryType.EBayTemplateRepository,
                    GetCacheEnabled( RepositoryType.EBayTemplateRepository ) );
            }
        }

        public static IGiftCertificateRepository GiftCertificateRepository
        {
            get
            {
                return ( IGiftCertificateRepository ) GetValue(
                    RepositoryType.GiftCertificateRepository,
                    GetCacheEnabled( RepositoryType.GiftCertificateRepository ) );
            }
        }

        public static IGiftRegistryItemRepository GiftRegistryItemRepository
        {
            get
            {
                return ( IGiftRegistryItemRepository ) GetValue(
                    RepositoryType.GiftRegistryItemRepository,
                    GetCacheEnabled( RepositoryType.GiftRegistryItemRepository ) );
            }
        }

        public static IGiftRegistryRepository GiftRegistryRepository
        {
            get
            {
                return ( IGiftRegistryRepository ) GetValue(
                    RepositoryType.GiftRegistryRepository,
                    GetCacheEnabled( RepositoryType.GiftRegistryRepository ) );
            }
        }

        public static IGoogleFeedMappingRepository GoogleFeedMappingRepository
        {
            get
            {
                return ( IGoogleFeedMappingRepository ) GetValue(
                    RepositoryType.GoogleFeedMappingRepository,
                    GetCacheEnabled( RepositoryType.GoogleFeedMappingRepository ) );
            }
        }

        public static IGoogleFeedTagRepository GoogleFeedTagRepository
        {
            get
            {
                return ( IGoogleFeedTagRepository ) GetValue(
                    RepositoryType.GoogleFeedTagRepository,
                    GetCacheEnabled( RepositoryType.GoogleFeedTagRepository ) );
            }
        }

        public static IGoogleFeedTagValueRepository GoogleFeedTagValueRepository
        {
            get
            {
                return ( IGoogleFeedTagValueRepository ) GetValue(
                    RepositoryType.GoogleFeedTagValueRepository,
                    GetCacheEnabled( RepositoryType.GoogleFeedTagValueRepository ) );
            }
        }

        public static IHelpRepository HelpRepository
        {
            get
            {
                return ( IHelpRepository ) GetValue(
                    RepositoryType.HelpRepository,
                    GetCacheEnabled( RepositoryType.HelpRepository ) );
            }
        }

        public static IManufacturerRepository ManufacturerRepository
        {
            get
            {
                return (IManufacturerRepository) GetValue(
                    RepositoryType.ManufacturerRepository,
                    GetCacheEnabled( RepositoryType.ManufacturerRepository ) );
            }
        }
        public static INewsLetterRepository NewsLetterRepository
        {
            get
            {
                return ( INewsLetterRepository ) GetValue(
                    RepositoryType.NewsLetterRepository,
                    GetCacheEnabled( RepositoryType.NewsLetterRepository ) );
            }
        }

        public static INewsRepository NewsRepository
        {
            get
            {
                return ( INewsRepository ) GetValue(
                    RepositoryType.NewsRepository,
                    GetCacheEnabled( RepositoryType.NewsRepository ) );
            }
        }

        public static IOptionGroupRepository OptionGroupRepository
        {
            get
            {
                return ( IOptionGroupRepository ) GetValue(
                    RepositoryType.OptionGroupRepository,
                    GetCacheEnabled( RepositoryType.OptionGroupRepository ) );
            }
        }

        public static IOptionItemRepository OptionItemRepository
        {
            get
            {
                return ( IOptionItemRepository ) GetValue(
                    RepositoryType.OptionItemRepository,
                    GetCacheEnabled( RepositoryType.OptionItemRepository ) );
            }
        }

        public static IOrderRepository OrderRepository
        {
            get
            {
                return ( IOrderRepository ) GetValue(
                    RepositoryType.OrderRepository,
                    GetCacheEnabled( RepositoryType.OrderRepository ) );
            }
        }

        public static IOrderItemRepository OrderItemRepository
        {
            get
            {
                return ( IOrderItemRepository ) GetValue(
                    RepositoryType.OrderItemRepository,
                    GetCacheEnabled( RepositoryType.OrderItemRepository ) );
            }
        }

        public static IPaymentLogRepository PaymentLogRepository
        {
            get
            {
                return ( IPaymentLogRepository ) GetValue(
                    RepositoryType.PaymentLogRepository,
                    GetCacheEnabled( RepositoryType.PaymentLogRepository ) );
            }
        }

        public static IPaymentOptionRepository PaymentOptionRepository
        {
            get
            {
                return ( IPaymentOptionRepository ) GetValue(
                    RepositoryType.PaymentOptionRepository,
                    GetCacheEnabled( RepositoryType.PaymentOptionRepository ) );
            }
        }

        public static IPayPalIpnRepository PayPalIpnRepository
        {
            get
            {
                return ( IPayPalIpnRepository ) GetValue(
                    RepositoryType.PayPalIpnRepository,
                    GetCacheEnabled( RepositoryType.PayPalIpnRepository ) );
            }
        }

        public static IProductKitGroupRepository ProductKitGroupRepository
        {
            get
            {
                return ( IProductKitGroupRepository ) GetValue(
                    RepositoryType.ProductKitGroupRepository,
                    GetCacheEnabled( RepositoryType.ProductKitGroupRepository ) );
            }
        }

        public static IProductRepository ProductRepository
        {
            get
            {
                return ( IProductRepository ) GetValue(
                    RepositoryType.ProductRepository,
                    GetCacheEnabled( RepositoryType.ProductRepository ) );
            }
        }

        public static IPromotionGroupRepository PromotionGroupRepository
        {
            get
            {
                return ( IPromotionGroupRepository ) GetValue(
                    RepositoryType.PromotionGroupRepository,
                    GetCacheEnabled( RepositoryType.PromotionGroupRepository ) );
            }
        }

        public static IPromotionProductRepository PromotionProductRepository
        {
            get
            {
                return ( IPromotionProductRepository ) GetValue(
                    RepositoryType.PromotionProductRepository,
                    GetCacheEnabled( RepositoryType.PromotionProductRepository ) );
            }
        }

        public static IPromotionSubGroupRepository PromotionSubGroupRepository
        {
            get
            {
                return ( IPromotionSubGroupRepository ) GetValue(
                    RepositoryType.PromotionSubGroupRepository,
                    GetCacheEnabled( RepositoryType.PromotionSubGroupRepository ) );
            }
        }

        public static IQuickBooksLogRepository QuickBooksLogRepository
        {
            get
            {
                return ( IQuickBooksLogRepository ) GetValue(
                    RepositoryType.QuickBooksLogRepository,
                    GetCacheEnabled( RepositoryType.QuickBooksLogRepository ) );
            }
        }

        public static IRecurringProfileRepository RecurringProfileRepository
        {
            get
            {
                return ( IRecurringProfileRepository ) GetValue(
                    RepositoryType.RecurringProfileRepository,
                    GetCacheEnabled( RepositoryType.RecurringProfileRepository ) );
            }
        }

        public static IRmaRepository RmaRepository
        {
            get
            {
                return ( IRmaRepository ) GetValue(
                    RepositoryType.RmaRepository,
                    GetCacheEnabled( RepositoryType.RmaRepository ) );
            }
        }

        public static IRmaActionRepository RmaActionRepository
        {
            get
            {
                return ( IRmaActionRepository ) GetValue(
                    RepositoryType.RmaActionRepository,
                    GetCacheEnabled( RepositoryType.RmaActionRepository ) );
            }
        }

        public static IShippingOptionRepository ShippingOptionRepository
        {
            get
            {
                return ( IShippingOptionRepository ) GetValue(
                    RepositoryType.ShippingOptionRepository,
                    GetCacheEnabled( RepositoryType.ShippingOptionRepository ) );
            }
        }

        public static IShippingOptionTypeRepository ShippingOptionTypeRepository
        {
            get
            {
                return ( IShippingOptionTypeRepository ) GetValue(
                    RepositoryType.ShippingOptionTypeRepository,
                    GetCacheEnabled( RepositoryType.ShippingOptionTypeRepository ) );
            }
        }

        public static IShippingWeightRateRepository ShippingWeightRateRepository
        {
            get
            {
                return ( IShippingWeightRateRepository ) GetValue(
                    RepositoryType.ShippingWeightRateRepository,
                    GetCacheEnabled( RepositoryType.ShippingWeightRateRepository ) );
            }
        }

        public static IShippingOrderTotalRateRepository ShippingOrderTotalRateRepository
        {
            get
            {
                return ( IShippingOrderTotalRateRepository ) GetValue(
                    RepositoryType.ShippingOrderTotalRateRepository,
                    GetCacheEnabled( RepositoryType.ShippingOrderTotalRateRepository ) );
            }
        }

        public static IShippingZoneGroupRepository ShippingZoneGroupRepository
        {
            get
            {
                return ( IShippingZoneGroupRepository ) GetValue(
                    RepositoryType.ShippingZoneGroupRepository,
                    GetCacheEnabled( RepositoryType.ShippingZoneGroupRepository ) );
            }
        }

        public static ISpecificationGroupRepository SpecificationGroupRepository
        {
            get
            {
                return ( ISpecificationGroupRepository ) GetValue(
                    RepositoryType.SpecificationGroupRepository,
                    GetCacheEnabled( RepositoryType.SpecificationGroupRepository ) );
            }
        }

        public static ISpecificationItemRepository SpecificationItemRepository
        {
            get
            {
                return ( ISpecificationItemRepository ) GetValue(
                    RepositoryType.SpecificationItemRepository,
                    GetCacheEnabled( RepositoryType.SpecificationItemRepository ) );
            }
        }

        public static ISpecificationItemValueRepository SpecificationItemValueRepository
        {
            get
            {
                return ( ISpecificationItemValueRepository ) GetValue(
                    RepositoryType.SpecificationItemValueRepository,
                    GetCacheEnabled( RepositoryType.SpecificationItemValueRepository ) );
            }
        }

        public static IStateRepository StateRepository
        {
            get
            {
                return ( IStateRepository ) GetValue(
                    RepositoryType.StateRepository,
                    GetCacheEnabled( RepositoryType.StateRepository ) );
            }
        }

        public static ISubscriptionLevelRepository SubscriptionLevelRepository
        {
            get
            {
                return ( ISubscriptionLevelRepository ) GetValue(
                    RepositoryType.SubscriptionLevelRepository,
                    GetCacheEnabled( RepositoryType.SubscriptionLevelRepository ) );
            }
        }

        public static IStoreRepository StoreRepository
        {
            get
            {
                return ( IStoreRepository ) GetValue(
                    RepositoryType.StoreRepository,
                    GetCacheEnabled( RepositoryType.StoreRepository ) );
            }
        }

        public static ITaxClassRepository TaxClassRepository
        {
            get
            {
                return ( ITaxClassRepository ) GetValue(
                    RepositoryType.TaxClassRepository,
                    GetCacheEnabled( RepositoryType.TaxClassRepository ) );
            }
        }

        public static IWishListRepository WishListRepository
        {
            get
            {
                return ( IWishListRepository ) GetValue(
                    RepositoryType.WishListRepository,
                    GetCacheEnabled( RepositoryType.WishListRepository ) );
            }
        }

        #endregion


        #region Public Methods

        public static void ClearConfigurationCache()
        {
            _configurations = null;
            _applicationSettings = null;
        }

        public static void SetUpRepository( RepositoryType type, object cached, object nonCached )
        {
            SetValue( type, true, cached );
            SetValue( type, false, nonCached );
        }

        public static bool GetCacheEnabled( RepositoryType type )
        {
            if ( !_cacheEnabledStates.ContainsKey( type ) )
                _cacheEnabledStates[ type ] = true;

            return _cacheEnabledStates[ type ];
        }

        public static void SetCacheEnabled( RepositoryType type, bool cacheEnabled )
        {
            _cacheEnabledStates[ type ] = cacheEnabled;
        }

        public static void EnableConfigurations( bool enabled )
        {
            _configEnabled = enabled;
            ClearConfigurationCache();
        }

        public static void SetStoreRetriever( IStoreRetriever storeRetriever )
        {
            _storeRetriever = storeRetriever;
        }
        #endregion
    }
}
